import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { RatesMinMaxService } from '../services/rates-min-max.service';
import { getCacheData } from '../utils/helper';
import { AgentWiseCommissionService } from '../services/agent-wise-commission.service';
import { CountryWiseCommissionService } from '../services/country-wise-commission.service';

@Component({
  selector: 'app-agent-wise-commission',
  templateUrl: './agent-wise-commission.component.html',
  styleUrls: ['./agent-wise-commission.component.css'],
})
export class AgentWiseCommissionComponent {
  @ViewChild('manageAgentWise') manageAgentWiseModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  currencies: any = [];
  selectedCurrency: any = 'All';

  sendCountries: any = [];
  selectedSendCountry: any = 'All';

  receiveCountries: any = [];
  selectedReceiveCountry: any = 'All';

  modeofPayments: any = [];
  selectedModeofPayment: any = 'All';

  sendAgents: any = [];
  selectedSendAgent: any = 'All';

  receiveAgents: any = [];
  selectedReceiveAgent: any = 'All';

  subAgents: any = [];
  selectedSubAgent: any = 'None';

  agentWiseCommission: any = [];

  sendInsertCountries: any = [];
  sendInsertAgents: any = [];
  modeofPaymentsInsert: any = [];
  sendInsertCurrencies: any = [];
  receiveInsertCountries: any = [];
  receiveInsertCurrencies: any = [];
  receivingInsertAgents: any = [];
  receivingSubAgents: any = [];
  totalCommissionTypes: any = [];
  sendCommissionTypes: any = [];
  receiveCommissionTypes: any = [];
  sendCurrencyTypes: any = [];
  receiveCurrencyTypes: any = [];
  rates: any = [];

  manageAgentWiseForm = new FormGroup({
    ID: new FormControl(),

    PayInCountryID: new FormControl('None', Validators.required),
    Agent_ID: new FormControl('None', Validators.required),
    ModeOfPayment_ID: new FormControl('None', Validators.required),
    PayInCurrencyID: new FormControl('None', Validators.required),
    PayOutCountryID: new FormControl('None', Validators.required),
    PayOutCurrencyID: new FormControl('None', Validators.required),
    RecAgent_ID: new FormControl('None', Validators.required),
    RecSubAgent_ID: new FormControl('None', Validators.required),
    RangeCurrencyType: new FormControl('None', Validators.required),
    StartRange: new FormControl('', Validators.required),
    EndRange: new FormControl('', Validators.required),
    BaseToBaseCommissionAmount: new FormControl('', Validators.required),
    PayCommissionType: new FormControl('None', Validators.required),
    PayInCommissionType: new FormControl('None', Validators.required),
    PayInCurrencyType: new FormControl('None', Validators.required),
    PayInCommissionAmount: new FormControl('None', Validators.required),
    PayOutCommissionType: new FormControl('None', Validators.required),
    PayOutCurrencyType: new FormControl('None', Validators.required),
    PayOutCommissionAmount: new FormControl('', Validators.required),
  });

  constructor(
    private agentWiseService: AgentWiseCommissionService,
    private countryCommissionService: CountryWiseCommissionService,
    private ratesService: RatesMinMaxService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendCountry();
    this.getModeofPayment();
    this.getSendCountryInsert();
    this.getReceiveCountryInsert();
    this.getTotalCommissionType();
  }
  get disabledInput() {
    return this.checkType == 'View' ? true : false;
  }
  getSendCountry() {
    this.agentWiseService
      .getSendCountry(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.sendCountries = res;
        },
        error: () => {
          this.sendCountries = [];
        },
        complete: () => {},
      });
  }

  getReceiveCountry(payInCountryId = 0) {
    this.agentWiseService
      .getReceiveCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.receiveCountries = res;
        },
        error: () => {
          this.receiveCountries = [];
        },
        complete: () => {},
      });
  }

  getModeofPayment() {
    this.agentWiseService
      .getModeofPayment(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: () => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  getSendAgent(payInCountryId = 0) {
    this.agentWiseService
      .getSendAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendAgents = res;
        },
        error: () => {
          this.sendAgents = [];
        },
        complete: () => {},
      });
  }

  onSendCountryChange() {
    if (this.selectedSendCountry != 'All') {
      this.getReceiveCountry(this.selectedSendCountry);
      this.getSendAgent(this.selectedSendCountry);
    }
  }

  getReceiveAgent(payOutCountryId = 0) {
    this.agentWiseService
      .getReceiveAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        payOutCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.receiveAgents = res;
        },
        error: () => {
          this.receiveAgents = [];
        },
        complete: () => {},
      });
  }

  getSubAgent(receiveAgentId = 0) {
    this.agentWiseService
      .getSubAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        receiveAgentId
      )
      .subscribe({
        next: (res: any) => {
          this.subAgents = res;
        },
        error: () => {
          this.subAgents = [];
        },
        complete: () => {},
      });
  }

  onReceiveCountryChange() {
    if (this.selectedReceiveCountry != 'All') {
      this.getReceiveAgent(this.selectedReceiveCountry);
    }
  }

  onReceiveAgentChange() {
    if (this.selectedReceiveAgent != 'All') {
      this.getSubAgent(this.selectedReceiveAgent);
    }
  }

  getAllAgentWiseCommission(
    payInCountryId: any = 0,
    payOutCountryId: any = 0,
    agentId: any = 0,
    receiveAgentId: any = 0,
    modeofPaymentId: any = 0,
    subAgentId: any
  ) {
    this.agentWiseService
      .getAllAgentWiseCommission(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId,
        payOutCountryId,
        agentId,
        receiveAgentId,
        modeofPaymentId,
        this.selectedSubAgent !== 'None' ? subAgentId : '-'
      )
      .subscribe({
        next: (res: any) => {
          this.agentWiseCommission = res;
        },
        error: () => {
          this.agentWiseCommission = [];
        },
        complete: () => {},
      });
  }

  searchAgentWiseCommission() {
    let payInCountryId: number | null = 0;
    let payOutCountryId: number | null = 0;
    let agentId: number | null = 0;
    let receiveAgentId: number | null = 0;
    let modeofPaymentId: number | null = 0;
    let subAgentId: number | null = 0;
    if (this.selectedSendCountry != 'All') {
      payInCountryId = this.selectedSendCountry;
    }
    if (this.selectedReceiveCountry != 'All') {
      payOutCountryId = this.selectedReceiveCountry;
    }
    if (this.selectedSendAgent != 'All') {
      agentId = this.selectedSendAgent;
    }
    if (this.selectedReceiveAgent != 'All') {
      receiveAgentId = this.selectedReceiveAgent;
    }
    if (this.selectedModeofPayment != 'All') {
      modeofPaymentId = this.selectedModeofPayment;
    }
    if (this.selectedSubAgent != 'All') {
      subAgentId = this.selectedSubAgent;
    }

    this.getAllAgentWiseCommission(
      payInCountryId,
      payOutCountryId,
      agentId,
      receiveAgentId,
      modeofPaymentId,
      subAgentId
    );
  }

  get f() {
    return this.manageAgentWiseForm.value;
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openAgentWiseModal(type: any) {
    console.log('type: ', type);
    this.type = type;

    if (type === 'add') {
      this.manageAgentWiseForm.reset({
        PayInCountryID: 'None',
        PayInCurrencyID: 'None',
        Agent_ID: 'None',
        ModeOfPayment_ID: 'None',
        PayOutCountryID: 'None',
        PayOutCurrencyID: 'None',
        RecAgent_ID: 'None',
        RecSubAgent_ID: 'None',
        RangeCurrencyType: 'None',
        StartRange: '',
        EndRange: '',
        BaseToBaseCommissionAmount: '',
        PayCommissionType: 'None',
        PayInCommissionType: 'None',
        PayOutCommissionType: 'None',
        PayInCurrencyType: 'None',
        PayInCommissionAmount: '',
        PayOutCurrencyType: 'None',
        PayOutCommissionAmount: '',
      });
    }

    this.openModal(this.manageAgentWiseModal);
  }

  addAgentCommission() {
    if (!this.manageAgentWiseForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryID == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.PayInCurrencyID == 'None') {
      alert('Please Select Send Currency!');
      return;
    }

    if (this.f.PayOutCountryID == 'None') {
      alert('Please Select Receive Country!');
      return;
    }
    if (this.f.PayOutCurrencyID == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }
    if (this.f.ModeOfPayment_ID == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (this.f.RecAgent_ID == 'None') {
      alert('Please Select Receiving Agent!');
      return;
    }

    if (this.f.RecSubAgent_ID == 'None') {
      alert('Please Select Receiving Sub Agent!');
      return;
    }

    if (this.f.RangeCurrencyType == 'None') {
      alert('Please Select Range Currency Type!');
      return;
    }

    if (this.f.EndRange < this.f.StartRange) {
      alert('End Range should be greater than Start Range!');
      return;
    }
    if (this.f.PayCommissionType == 'None') {
      alert('Please Select Total Commission Type!');
      return;
    }
    if (this.f.PayInCommissionType == 'None') {
      alert('Please Select Send Commission Type!');
      return;
    }
    if (this.f.PayInCurrencyType == 'None') {
      alert('Please Select Send Currency Type!');
      return;
    }
    if (this.f.PayOutCommissionType == 'None') {
      alert('Please Select Receive Commission Type!');
      return;
    }
    if (this.f.PayOutCurrencyType == 'None') {
      alert('Please Select Receive Currency Type!');
      return;
    }

    let payload: any;
    payload = {
      PayInCountryID: this.f.PayInCountryID,
      Agent_ID: this.f.Agent_ID,
      PayInCurrencyID: this.f.PayInCurrencyID,
      PayOutCountryID: this.f.PayOutCountryID,
      PayOutCurrencyID: this.f.PayOutCurrencyID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      StartRange: this.f.StartRange,
      EndRange: this.f.EndRange,
      BaseToBaseCommissionAmount: this.f.BaseToBaseCommissionAmount,
      PayCommissionType: this.f.PayCommissionType,
      PayInCommissionType: this.f.PayInCommissionType,
      PayInCurrencyType: this.f.PayInCurrencyType,
      PayInCommissionAmount: this.f.PayInCommissionAmount,
      PayOutCommissionType: this.f.PayOutCommissionType,
      PayOutCurrencyType: this.f.PayOutCurrencyType,
      PayOutCommissionAmount: this.f.PayOutCommissionAmount,
      EnteredBy: getCacheData('username'),
      RecAgent_ID: this.f.RecAgent_ID,
      RangeCurrencyType: this.f.RangeCurrencyType,
      RecSubAgent_ID: this.f.RangeCurrencyType,
    };

    console.log('this.type: ', this.type);
    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: this.f.ID,
      };
    }

    if (this.type === 'view') {
      console.log('this.f.ID: ', this.f.ID);
      this.approveAgentWiseCommission(1, this.f.ID);
      return;
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addAgentWiseCommission'
        : 'updateAgentWiseCommission';

    this.agentWiseService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  approveAgentWiseCommission(moduleID: any = 1, Id: any) {
    this.agentWiseService.approveAgentWiseCommission(moduleID, Id).subscribe({
      next: (res: any) => {
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editRates(rate: any) {
    // this.manageAgentWiseForm.patchValue({
    //   ID: rate?.ID,
    //   CurrencyId: currencyId,
    //   RatePer: rate?.RatePER,
    // });
    this.openAgentWiseModal('edit');
  }

  isRatesPerValid(rate: number): boolean {
    return !!rate;
  }

  updateRates(): void {
    const updatedRates = this.rates
      .filter((con) => con.isChecked)
      .map((con) => {
        return {
          IsCheckEdit: true,
          IsActive: true,
          Id: con?.ID,
          RatePer: con?.RatePER,
          UserID: 0,
        };
      });
    if (updatedRates?.length) {
      this.ratesService
        .updateRates(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedRates
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
          },
          error: (err) => {
            this.modalService.hide();
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getSendCountryInsert() {
    this.agentWiseService
      .getSendCountryInsert(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendInsertCountries = res;
          this.getSendAgentInsert(res?.[0]?.CountryId);
          this.getSendCurrencyInsert(res?.[0]?.CountryId);
        },
        error: () => {
          this.sendInsertCountries = [];
        },
        complete: () => {},
      });
  }
  async editCountryWiseCommission(country: any, type: any) {
    this.getAgentWiseCommissionDataById(country.ID, type);
  }

  getAgentWiseCommissionDataById(id: any, type) {
    this.agentWiseService.getAgentWiseCommissionDataById(id).subscribe({
      next: (res: any) => {
        const getAgentWiseData = res[0];

        this.filledValue(getAgentWiseData, type);
      },
      error: () => {},
      complete: () => {},
    });
  }

  get checkType() {
    let type = this.type;
    switch (type) {
      case 'add':
        return (type = 'Add');
      case 'edit':
        return (type = 'Update');
      case 'view':
        return (type = 'View');
      default:
        return type;
    }
  }

  filledValue(data: any, type) {
    const PayInCountryID = this.sendInsertCountries.find(
      (c) => c.CountryId == data.PayInCountryId
    )?.CountryId;

    const payOutCountryID = this.receiveCountries.find(
      (c) => c.PayOutCountryId == data.PayOutCountryId
    )?.CountryName;

    const payInCurrencyId = this.sendInsertCurrencies.find(
      (c) => c.Currency_Id == data.PayInCurrencyID
    )?.CurrencyShortName;

    const payOutCurrencyId = this.receiveInsertCurrencies.find(
      (c) => c.Currency_Id == data.PayOutCurrencyID
    )?.CurrencyShortName;

    const modeofPaymentId = this.modeofPaymentsInsert.find(
      (c) => c.ModeOfPayment_ID == data.ModeOfPayment_ID
    )?.ModeOfPayment_Name;

    const payCommissionType = this.totalCommissionTypes.find(
      (c) => c.ID == data.PayCommissionType
    )?.CommissionType;

    const PayInCommissionType = this.sendCommissionTypes.find(
      (c) => c.ID == data.PayInCommissionType
    )?.CommissionType;

    const payInCurrencyType = this.sendCurrencyTypes.find(
      (c) => c.ID == data.PayInCurrencyType
    )?.CurrencyType;

    const payOutCommissionType = this.receiveCommissionTypes.find(
      (c) => c.ID == data.PayOutCommissionType
    )?.CommissionType;

    const payOutCurrencyType = this.receiveCurrencyTypes.find(
      (c) => c.ID == data.PayOutCurrencyType
    )?.CurrencyType;

    const RangeCurrencyType = this.receiveInsertCurrencies.find(
      (c) => c.Currency_Id == data.RangeCurrencyType
    )?.CurrencyShortName;

    this.receivingInsertAgents = [{ AgentName: 'LM', Agent_ID: 9 }];

    const RecAgent_ID = this.receivingInsertAgents.find(
      (c) => c.Agent_ID == data.ReceivingAgent
    )?.AgentName;

    this.manageAgentWiseForm.patchValue({
      ID: data?.ID,
      PayInCountryID: PayInCountryID,
      PayInCurrencyID: payInCurrencyId,
      PayOutCountryID: payOutCountryID,
      PayOutCurrencyID: payOutCurrencyId,
      ModeOfPayment_ID: modeofPaymentId,
      StartRange: data?.StartRange,
      EndRange: data?.EndRange,
      BaseToBaseCommissionAmount: data?.BaseToBaseCommissionAmount,
      PayCommissionType: payCommissionType,
      PayInCommissionType: PayInCommissionType,
      PayInCurrencyType: payInCurrencyType,
      PayInCommissionAmount: data?.PayInCommissionAmount,
      PayOutCommissionType: payOutCommissionType,
      PayOutCurrencyType: payOutCurrencyType,
      PayOutCommissionAmount: data?.PayOutCommissionAmount,
      Agent_ID: data?.Agent_ID,
      RecAgent_ID: RecAgent_ID,
      RangeCurrencyType: RangeCurrencyType,
    });
    this.openAgentWiseModal(type);
  }

  getSendAgentInsert(payInCountryId = 0) {
    this.agentWiseService
      .getSendAgentInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendInsertAgents = res;
          this.getModeofPaymentInsert(res?.[0]?.Agent_ID);
          this.getSendCurrencyType();
        },
        error: () => {
          this.sendInsertAgents = [];
        },
        complete: () => {},
      });
  }

  getModeofPaymentInsert(agentId = 0) {
    this.agentWiseService
      .getModeofPaymentInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPaymentsInsert = res;
        },
        error: () => {
          this.modeofPaymentsInsert = [];
        },
        complete: () => {},
      });
  }

  getSendCurrencyInsert(countryId: any = 0) {
    this.agentWiseService
      .getSendCurrencyInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendInsertCurrencies = res;
        },
        error: () => {
          this.sendInsertCurrencies = [];
        },
        complete: () => {},
      });
  }

  onSendInsertCountryChange(countryId: any) {
    if (countryId != 'None') {
      this.getSendAgentInsert(countryId);
    }
  }

  onSendInsertAgentChange(agentId: any) {
    if (agentId != 'None') {
      this.getModeofPaymentInsert(agentId);
    }
  }

  onSendInsertModeofPayment(modeofPaymentId: any) {
    if (modeofPaymentId != 'None') {
      this.getSendCurrencyInsert(this.f.PayInCountryID);
    }
  }

  getReceiveCountryInsert() {
    this.agentWiseService
      .getReceiveCountryInsert(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.receiveInsertCountries = res;
          this.getReceiveCurrencyInsert(res?.[0]?.CountryId);
        },
        error: () => {
          this.receiveInsertCountries = [];
        },
        complete: () => {},
      });
  }

  onReceiveCountryInsertChange(countryId: any) {
    if (countryId != 'None') {
      this.getReceiveCurrencyInsert(countryId);
    }
  }

  getReceiveCurrencyInsert(payOutCountryId = 0) {
    this.agentWiseService
      .getReceiveCurrencyInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        payOutCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.receiveInsertCurrencies = res;
          this.getReceiveAgentInsert(res?.[0]?.Currency_Id, payOutCountryId);
        },
        error: () => {
          this.receiveInsertCurrencies = [];
        },
        complete: () => {},
      });
  }

  onReceiveCurrencyInsertChange(currencyId: any) {
    if (currencyId != 'None') {
      this.manageAgentWiseForm.patchValue({
        RangeCurrencyType: currencyId,
      });
      this.getReceiveAgentInsert(currencyId, this.f.PayOutCountryID);
    }
  }

  getReceiveAgentInsert(receiveCurrencyId: any = 0, payOutCountryId: any = 0) {
    this.agentWiseService
      .getReceiveAgentInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        receiveCurrencyId,
        payOutCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingInsertAgents = res;
          this.getReceiveSubAgentInsert(res?.[0]?.Agent_ID);
        },
        error: () => {
          this.receivingInsertAgents = [];
        },
        complete: () => {},
      });
  }

  onReceiveAgentInsertChange(agentId: any) {
    if (agentId != 'None') {
      this.getReceiveSubAgentInsert(agentId);
    }
  }

  getReceiveSubAgentInsert(agentId: any = 0) {
    this.agentWiseService
      .getReceiveSubAgentInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingSubAgents = res;
        },
        error: () => {
          this.receivingSubAgents = [];
        },
        complete: () => {},
      });
  }

  getTotalCommissionType() {
    this.countryCommissionService
      .getTotalCommissionType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.totalCommissionTypes = res;
          this.getSendCommissionType();
          this.getReceiveCommissionType();
        },
        error: () => {
          this.totalCommissionTypes = [];
        },
        complete: () => {},
      });
  }

  getSendCommissionType() {
    if (this.totalCommissionTypes?.length) {
      this.sendCommissionTypes = this.totalCommissionTypes;
    }
  }

  getReceiveCommissionType() {
    if (this.totalCommissionTypes?.length) {
      this.receiveCommissionTypes = this.totalCommissionTypes;
    }
  }

  getSendCurrencyType() {
    this.countryCommissionService
      .getCurrencyType(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.sendCurrencyTypes = res;
          this.getReceiveCurrencyType();
        },
        error: () => {
          this.sendCurrencyTypes = [];
        },
        complete: () => {},
      });
  }

  getReceiveCurrencyType() {
    if (this.sendCurrencyTypes?.length) {
      this.receiveCurrencyTypes = this.sendCurrencyTypes;
    }
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  get validUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 6 && gUserGroupNumber === 1;
    }

    return false;
  }

  get validAuthorizeUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 1 && gUserGroupNumber === 1;
    }

    return false;
  }

  update() {
    const deletedCountries = this.getDeletedCountries();
    const updatedCountries = this.getUpdatedAgentWiseCommission();

    this.deleteAgentWiseCommission(deletedCountries);
    this.updatedAgentWiseCommission(updatedCountries);
  }

  getUpdatedAgentWiseCommission() {
    const updatedCommissions = this.agentWiseCommission
      .filter((con) => con.isChecked)
      .map((con) => ({
        ID: con?.ID,
        StartRange: Number(con?.StartRange),
        EndRange: Number(con?.EndRange),
        BaseToBaseCommissionAmount: Number(con?.BaseToBaseCommissionAmount),
        SendCommissionAmount: Number(con?.PayInCommissionAmount),
        ReceiveCommissionAmount: Number(con?.PayOutCommissionAmount),
        UserID: 'string',
        UpdatedStartRange: Number(con?.StartRange),
        UpdatedEndRange: Number(con?.EndRange),
        UpdatedBaseToBaseCommissionAmount: Number(
          con?.BaseToBaseCommissionAmount
        ),
        UpdatedSendCommissionAmount: Number(con?.PayInCommissionAmount),
        UpdatedReceiveCommissionAmount: Number(con?.PayOutCommissionAmount),
        RecSubAgent: 0,
      }));

    return {
      IsValidUser: this.validUser,
      IsValidAuthorizeUser: this.validAuthorizeUser,
      AgentGridUpdateRequestList: updatedCommissions,
    };
  }

  isValidEndRange(start_range: number, end_range: number): boolean {
    return Number(start_range) < Number(end_range);
  }

  updatedAgentWiseCommission(updatedCountries) {
    if (updatedCountries?.AgentGridUpdateRequestList?.length) {
      this.agentWiseService
        .updateAgentWiseCommissionGrid(updatedCountries)
        .subscribe({
          next: (res: any) => {
            this.searchAgentWiseCommission();
            alert('Agent Wise Commission Update Successfully.');
          },
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }

  getDeletedCountries() {
    return this.agentWiseCommission
      .filter((con) => con.isDeleted)
      .map((con) => ({ ID: con?.ID, RecSubAgent: 0 }));
  }

  deleteAgentWiseCommission(deletedCountries) {
    if (deletedCountries?.length) {
      this.agentWiseService
        .deleteAgentWiseCommission(deletedCountries)
        .subscribe({
          next: (res: any) => {
            this.searchAgentWiseCommission();
            alert('Agent Wise Commission Deleted Successfully.');
          },
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }
}
