import { Component } from '@angular/core';
import { CustomerRegistrationService } from '../services/customer-registration.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { EditCustomerRegistrationService } from '../services/edit-customer-registration.service';

@Component({
  selector: 'app-customer-registration',
  templateUrl: './customer-registration.component.html',
  styleUrls: ['./customer-registration.component.css'],
})
export class CustomerRegistrationComponent {
  occupationList: any = [];
  sourceList: any = [];
  productTypeList: any = [];
  nationalityList: any = [];
  genderList: any = [
    {
      id: 'M',
      value: 'Male',
    },
    {
      id: 'F',
      value: 'Female',
    },
  ];

  idTypeList: any = [];

  statusList: any = [
    {
      id: 1,
      value: 'Active',
    },
    {
      id: 0,
      value: 'InActive',
    },
  ];

  customerTypeList: any = [
    {
      id: 1,
      value: '---EASYPAY CUSTOMER---',
    },
    {
      id: 4,
      value: '---VIRGIN CUSTOMER---',
    },
  ];

  customerRegistrationForm = new FormGroup({
    CustomerType: new FormControl(1, Validators.required),
    CustomerNo: new FormControl('', Validators.required),
    CustomerAccount: new FormControl('', Validators.required),
    EasyPayExpiryDate: new FormControl(''),
    ProductCode: new FormControl(''),
    CustomerName: new FormControl(''),
    FirstName: new FormControl(''),
    CusMiddleName: new FormControl(''),
    CusLastName: new FormControl(''),
    CustomerAddress: new FormControl('', Validators.required),
    DOB: new FormControl('', Validators.required),
    Gender: new FormControl(''),
    MobilePrefix: new FormControl(''),
    MobileNumber: new FormControl('', Validators.required),
    Nationality: new FormControl('-Select-', Validators.required),
    IDNumber: new FormControl('', Validators.required),
    IDType: new FormControl(''),
    IDExpiryDate: new FormControl('', Validators.required),
    Sourceoffund: new FormControl('-Select Source Of Fund-'),
    TotalIncome: new FormControl(''),
    UserID: new FormControl(''),
    Occupation: new FormControl('-Select Occupation-', Validators.required),
    City: new FormControl('', Validators.required),
    BirthCountry: new FormControl('', Validators.required),
  });

  constructor(
    private customerRegistrationSvc: CustomerRegistrationService,
    private editCustomerRegistrationSvc: EditCustomerRegistrationService
  ) {}

  ngOnInit(): void {
    this.getOccupationList();
    this.getSourceList();
    this.getNationalityList();
    this.getIdTypeList();
  }

  getOccupationList() {
    this.customerRegistrationSvc.getOccupationList().subscribe({
      next: (res: any) => {
        this.occupationList = res;
      },
      error: (err) => {
        this.occupationList = [];
      },
      complete: () => {},
    });
  }

  getSourceList() {
    this.customerRegistrationSvc.getSourceList().subscribe({
      next: (res: any) => {
        this.sourceList = res;
      },
      error: (err) => {
        this.sourceList = [];
      },
      complete: () => {},
    });
  }

  getNationalityList() {
    this.customerRegistrationSvc.getNationalityList().subscribe({
      next: (res: any) => {
        this.nationalityList = res;
      },
      error: (err) => {
        this.nationalityList = [];
      },
      complete: () => {},
    });
  }

  getIdTypeList() {
    this.editCustomerRegistrationSvc.getIdTypeList().subscribe({
      next: (res: any) => {
        this.idTypeList = res?.[0]?.IDType;
      },
      error: (err) => {
        this.idTypeList = [];
      },
      complete: () => {},
    });
  }

  get f() {
    return this.customerRegistrationForm.value;
  }

  addCustomerRegistration() {
    if (!this.customerRegistrationForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (this.f.Nationality == '-Select-') {
      alert('Please Select Nationality.');
      return;
    }

    if (this.f.Occupation == '-Select Occupation-') {
      alert('Please Select Occupation.');
      return;
    }

    if (this.f.Sourceoffund == '-Select Source Of Fund-') {
      alert('Please Select Source of Fund.');
      return;
    }

    if (this.f.CustomerType != 1) {
      if (!this.f.FirstName) {
        alert('Please Fill First Name.');
        return;
      }
      if (!this.f.CusLastName) {
        alert('Please Fill Last Name.');
        return;
      }
      if (!this.f.IDType) {
        alert('Please Select ID Type.');
        return;
      }
    } else {
      if (!this.f.CustomerName) {
        alert('Please Fill Customer Name.');
        return;
      }
      if (!this.f.EasyPayExpiryDate) {
        alert('Please Fill Easy Pay Expiry Date.');
        return;
      }
    }

    let payload;
    payload = {
      CustomerType: this.f.CustomerType,
      CustomerNo: this.f.CustomerNo,
      CustomerAccount: this.f.CustomerAccount,
      CustomerAddress: this.f.CustomerAddress,
      ProductCode: '',
      DOB: this.f.DOB,
      Gender: this.f.Gender,
      MobileNumber: this.f.MobileNumber,
      Nationality: this.f.Nationality,
      IDNumber: this.f.IDNumber,
      IDExpiryDate: this.f.IDExpiryDate,
      Sourceoffund: this.f.Sourceoffund,
      TotalIncome: this.f.TotalIncome,
      Occupation: this.f.Occupation,
      City: this.f.City,
      BirthCountry: this.f.BirthCountry,
    };

    if (this.f.CustomerType != 1) {
      payload = {
        ...payload,
        FirstName: this.f.FirstName,
        CusMiddleName: this.f.CusMiddleName,
        CusLastName: this.f.CusLastName,
        MobilePrefix: '+966',
        IDType: this.f.IDType,
        Telephone: this.f.MobileNumber,
      };
    } else {
      payload = {
        ...payload,
        EasyPayExpiryDate: this.f.EasyPayExpiryDate,
        CustomerName: this.f.CustomerName,
      };
    }
    this.customerRegistrationSvc.addCustomerRegistration(payload).subscribe({
      next: (res: any) => {
        alert(`
        Customer Number - ${this.f.CustomerNo} registered successfully!`);
        this.customerRegistrationForm.reset({
          CustomerType: 1,
          Nationality: '-Select-',
          Sourceoffund: '-Select Source Of Fund-',
          Occupation: '-Select Occupation-',
        });
      },
      error: (err) => {
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  onNationalityChange() {
    {
      if (this.f.Nationality == '-Select-') {
        this.f.BirthCountry = '';
      } else {
        this.customerRegistrationForm.patchValue({
          BirthCountry: this.f.Nationality,
        });
      }
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
