import { Component } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { CustomerRegistrationService } from '../services/customer-registration.service';
import { EditCustomerRegistrationService } from '../services/edit-customer-registration.service';

@Component({
  selector: 'app-edit-customer-registration',
  templateUrl: './edit-customer-registration.component.html',
  styleUrls: ['./edit-customer-registration.component.css'],
})
export class EditCustomerRegistrationComponent {
  occupationList: any = [];
  sourceList: any = [];
  productTypeList: any = [];
  nationalityList: any = [];
  genderList: any = [
    {
      id: 'M',
      value: 'Male',
    },
    {
      id: 'F',
      value: 'Female',
    },
  ];

  idTypeList: any = [];

  statusList: any = [
    {
      id: 1,
      value: 'Active',
    },
    {
      id: 0,
      value: 'InActive',
    },
  ];

  customerRegistrationForm = new FormGroup({
    FirstName: new FormControl('', Validators.required),
    MiddleName: new FormControl(''),
    Occupation: new FormControl('-Select Occupation-', Validators.required),
    customertype: new FormControl(''),
    CustomerNo: new FormControl('', Validators.required),
    CustomerAccountNo: new FormControl('', Validators.required),
    EasyPayExpriyDate: new FormControl('', Validators.required),
    producttype: new FormControl('-Select-'),
    LastName: new FormControl('', Validators.required),
    CustomerAddress: new FormControl(''),
    dob: new FormControl('', Validators.required),
    Gender: new FormControl('-Select-'),
    telephone: new FormControl('', Validators.required),
    MobilePrefix: new FormControl(''),
    Nationality: new FormControl('-Select-', Validators.required),
    IDtype: new FormControl('-Select-'),
    Idnumber: new FormControl('', Validators.required),
    Idexpirydate: new FormControl('', Validators.required),
    Sourceoffund: new FormControl(
      '-Select Source Of Fund-',
      Validators.required
    ),
    TotalMonthlyIncome: new FormControl('', Validators.required),
    Status: new FormControl('-Select Status-'),
    City: new FormControl('', Validators.required),
    BirthCountry: new FormControl('', Validators.required),
    UserID: new FormControl(''),
    RegDateTime: new FormControl('', Validators.required),
  });

  constructor(
    private customerRegistrationSvc: CustomerRegistrationService,
    private editCustomerRegistrationSvc: EditCustomerRegistrationService
  ) {}

  ngOnInit(): void {
    this.getNationalityList();
    this.getProductTypeList();
    this.getOccupationList();
    this.getSourceList();
    this.getIdTypeList();
  }

  getOccupationList() {
    this.customerRegistrationSvc.getOccupationList().subscribe({
      next: (res: any) => {
        this.occupationList = res;
      },
      error: (err) => {
        this.occupationList = [];
      },
      complete: () => {},
    });
  }

  getSourceList() {
    this.customerRegistrationSvc.getSourceList().subscribe({
      next: (res: any) => {
        this.sourceList = res;
      },
      error: (err) => {
        this.sourceList = [];
      },
      complete: () => {},
    });
  }

  getNationalityList() {
    this.customerRegistrationSvc.getNationalityList().subscribe({
      next: (res: any) => {
        this.nationalityList = res;
      },
      error: (err) => {
        this.nationalityList = [];
      },
      complete: () => {},
    });
  }

  getProductTypeList() {
    this.editCustomerRegistrationSvc.getProductTypeList().subscribe({
      next: (res: any) => {
        this.productTypeList = res;
      },
      error: (err) => {
        this.productTypeList = [];
      },
      complete: () => {},
    });
  }

  getIdTypeList() {
    this.editCustomerRegistrationSvc.getIdTypeList().subscribe({
      next: (res: any) => {
        this.idTypeList = res?.[0]?.IDType;
      },
      error: (err) => {
        this.idTypeList = [];
      },
      complete: () => {},
    });
  }

  get f() {
    return this.customerRegistrationForm.value;
  }

  searchCustomer() {
    if (!this.f.CustomerNo) {
      alert('Please Fill Customer Number!');
      return;
    }
    this.editCustomerRegistrationSvc
      .getSearchByCustomerNumber(this.f.CustomerNo)
      .subscribe({
        next: (res: any) => {
          this.mapCustomerDetails(res?.[0]);
        },
        error: (err) => {},
        complete: () => {},
      });
  }

  mapCustomerDetails(customer: any) {
    if (customer) {
      const nationality = this.nationalityList.find(
        (n) => n.NShortName == customer?.Nationality
      )?.NShortName;

      const productType = this.productTypeList.find(
        (p) => p.ProductType == customer?.ProductType
      )?.ProductType;

      const cusGender = customer?.Gender.trim();
      const gender = this.genderList.find((g) => g.id == cusGender)?.id;

      const occupation = this.occupationList.find(
        (o) => o.OccupationID == customer?.Occupation
      )?.OccupationID;

      const idType = this.idTypeList.find(
        (i) => i.TypeID == customer?.IDType
      )?.TypeID;

      const status = this.statusList.find(
        (s) => s.id == customer?.IsActive
      )?.id;

      const source = this.sourceList.find(
        (s) => s.SourceOfIncome_ID == customer?.SourceOfFund
      )?.SourceOfIncome_ID;

      this.customerRegistrationForm.patchValue({
        CustomerNo: customer?.CustomerNo,
        FirstName: customer?.FirstName,
        LastName: customer?.LastName,
        CustomerAddress: customer?.Address,
        Nationality: nationality,
        producttype: productType,
        Gender: gender?.trim(),
        Idnumber: customer?.IDNumber,
        Idexpirydate: this.formatDate(customer?.IDExpiryDate),
        Occupation: occupation,
        BirthCountry: customer?.BirthCountry,
        TotalMonthlyIncome: customer?.TotalMonthlyIncome,
        MiddleName: customer?.MiddleName,
        customertype: customer?.CustomerTypeName,
        EasyPayExpriyDate: customer?.EasyPayExpiryDate,
        CustomerAccountNo: customer?.CustomerAccountNo,
        dob: this.formatDate(customer?.DateOfBirth),
        telephone: customer?.Telephone,
        IDtype: idType,
        RegDateTime: this.formatDate(customer?.RegDateTime),
        City: customer?.City,
        Sourceoffund: source,
        Status: status,
      });
    }
  }

  formatDate(inputDate: string): string {
    // Split the inputDate into parts (assuming it is in the format MM/dd/yyyy)
    let dateParts = inputDate.split('/');

    // Create a new Date object using the parts
    let formattedDate = new Date(
      `${dateParts[2]}-${dateParts[0]}-${dateParts[1]}`
    );

    // Format the date as "yyyy-MM-dd"
    return formattedDate.toISOString().split('T')[0];
  }

  updateCustomerRegistration() {
    if (!this.customerRegistrationForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (this.f.Nationality == '-Select-') {
      alert('Please Select Nationality.');
      return;
    }
    if (this.f.producttype == '-Select-') {
      alert('Please Select Product Type.');
      return;
    }
    if (this.f.Occupation == '-Select Occupation-') {
      alert('Please Select Occupation.');
      return;
    }
    if (this.f.IDtype == '-Select-') {
      alert('Please Select ID Type.');
      return;
    }
    if (this.f.Sourceoffund == '-Select Source Of Fund-') {
      alert('Please Select Source of Fund.');
      return;
    }

    let payload;
    payload = {
      CustomerNo: this.f.CustomerNo,
      FirstName: this.f.FirstName,
      LastName: this.f.LastName,
      CustomerAddress: this.f.CustomerAddress,
      Nationality: this.f.Nationality,
      producttype: this.f.producttype,
      Gender: this.f.Gender,
      Idnumber: this.f.Idnumber,
      Idexpirydate: this.f.Idexpirydate,
      Occupation: this.f.Occupation,
      BirthCountry: this.f.BirthCountry,
      TotalMonthlyIncome: this.f.TotalMonthlyIncome,
      MiddleName: this.f.MiddleName,
      customertype: this.f.customertype,
      EasyPayExpriyDate: this.f.EasyPayExpriyDate,
      CustomerAccountNo: this.f.CustomerAccountNo,
      dob: this.f.dob,
      telephone: this.f.telephone,
      IDtype: this.f.IDtype,
      RegDateTime: this.f.RegDateTime,
      City: this.f.City,
      Sourceoffund: this.f.Sourceoffund,
      Status: this.f.Status,
    };
    this.editCustomerRegistrationSvc
      .updateCustomerRegistration(payload)
      .subscribe({
        next: (res: any) => {},
        error: (err) => {},
        complete: () => {},
      });
  }

  resetCustomerRegistration() {
    this.customerRegistrationForm.reset();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
