import { DOCUMENT } from '@angular/common';
import { Component, Inject, OnInit } from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { AuthService } from '../services/auth.service';
import { Router } from '@angular/router';
import { setCacheData } from '../utils/helper';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css'],
})
export class LoginComponent implements OnInit {
  loginForm = new FormGroup({
    username: new FormControl('', Validators.required),
    password: new FormControl('', Validators.required),
  });

  showPassword = false;

  constructor(
    @Inject(DOCUMENT) private document: Document,
    private fb: FormBuilder,
    private authService: AuthService,
    private router: Router
  ) {
    this.document.body.classList.add('login-page');
  }
  ngOnInit(): void {}

  get f() {
    return this.loginForm.value;
  }

  submitForm() {
    if (this.loginForm.valid) {
      this.authService.login(this.f.username, this.f.password).subscribe({
        next: (res: any) => {
          if (res?.data?.AuthorizationToken) {
            setCacheData('admin_token', res?.data?.AuthorizationToken);
            setCacheData('username', res?.data?.G_UserName);
            setCacheData('G_UserGroup', res?.data?.G_UserGroup);
            setCacheData('SubUserGroup_Id', res?.data?.SubUserGroup_Id);

            this.router.navigate(['/panel/country-management']);
          }
        },
        error: (err) => {},
        complete: () => {},
      });
    }
  }

  togglePassword() {
    this.showPassword = !this.showPassword;
  }

  ngOnDestroy() {
    this.document.body.classList.remove('login-page');
  }
}
