import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { ManageCountryService } from '../manage-country.service';
import { ManageModeOfPaymentService } from '../services/manage-mode-of-payment.service';

@Component({
  selector: 'app-manage-mode-of-payment',
  templateUrl: './manage-mode-of-payment.component.html',
  styleUrls: ['./manage-mode-of-payment.component.css'],
})
export class ManageModeOfPaymentComponent {
  @ViewChild('manageModeofPayment') manageModeofPaymentModal;
  bsModalRef: BsModalRef;

  countries: any = [];
  filterCounteries: any = [];
  selectedCountry: any = 'All';

  type: any = 'add';

  modeofPayments: any = [];

  allPaymentModes: any = [];

  manageModeofPaymentForm = new FormGroup({
    ID: new FormControl(),
    CountryID: new FormControl('All', Validators.required),
    ModeOfPayment_ID: new FormControl('None', Validators.required),
    IsActive: new FormControl(true),
    Message: new FormControl('none'),
  });

  constructor(
    private countryService: ManageCountryService,
    private paymentModeService: ManageModeOfPaymentService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountries();
    this.getAllManageModeofPayment();
    this.getModeofPayments();
  }

  getAllManageModeofPayment(countryId = 0) {
    this.paymentModeService
      .getAllManageModeofPayment(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: (err) => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  getModeofPayments() {
    this.paymentModeService
      .getModeofPayments(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.allPaymentModes = res;
        },
        error: (err) => {
          this.allPaymentModes = [];
        },
        complete: () => {},
      });
  }

  getCountries() {
    this.countryService
      .getCountries(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.countries = res;
          this.filterCounteries = [...res];
        },
        error: (err) => {
          this.countries = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openModeofPaymentModal(type: any) {
    if (type == 'add') {
      this.manageModeofPaymentForm.reset({
        CountryID: this.selectedCountry,
        ModeOfPayment_ID: 'None',
        IsActive: false,
      });
    }
    this.type = type;
    this.openModal(this.manageModeofPaymentModal);
  }

  searchModeofPayment() {
    let countryId: number | null = 0;

    if (this.selectedCountry !== 'All') {
      countryId = this.selectedCountry;
    }

    this.getAllManageModeofPayment(countryId);
  }

  get f() {
    return this.manageModeofPaymentForm.value;
  }

  addModeofPayment() {
    if (!this.manageModeofPaymentForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.CountryID == 'All') {
      alert('Please Select Country!');
      return;
    }

    if (this.f.ModeOfPayment_ID == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    let payload;
    payload = {
      CountryID: this.f.CountryID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      IsActive: this.f.IsActive,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: 'none',
      UpdatedBy: 'none',
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: this.f.ID,
        UpdatedOn: new Date(),
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addModeofPayment' : 'updateModeofPayment';

    this.paymentModeService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllManageModeofPayment();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editModeofPayment(payment: any) {
    const countryId = this.countries.find(
      (p) => p.CountryName == payment.CountryName
    )?.CountryId;
    const modeofPaymentId = this.allPaymentModes.find(
      (p) => p.ModeOfPayment_Name == payment.ModeOfPayment
    )?.ModeOfPayment_ID;

    this.manageModeofPaymentForm.patchValue({
      ID: payment?.ID,
      CountryID: countryId,
      ModeOfPayment_ID: modeofPaymentId,
      IsActive: payment?.IsActive,
    });
    this.openModeofPaymentModal('edit');
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
