import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { getCacheData } from '../utils/helper';
import { ManagePurposeOfTransferService } from '../services/manage-purpose-of-transfer.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-manage-purpose-of-transfer',
  templateUrl: './manage-purpose-of-transfer.component.html',
  styleUrls: ['./manage-purpose-of-transfer.component.css'],
})
export class ManagePurposeOfTransferComponent implements OnInit {
  @ViewChild('managePurposeofTransfer') managePurposeofTransferModal;
  bsModalRef: BsModalRef;

  purposeOfTransfers: any = [];

  allPurposeOfTransfers: any = [];
  selectedPurposeOfTransfer = 'All';

  type: any = 'add';

  managePurposeofTransferForm = new FormGroup({
    PurposeOfTransfer_ID: new FormControl(),
    PurposeOfTransfer_Name: new FormControl('', Validators.required),
    Purpose_Value: new FormControl('', Validators.required),
    BaseTypeID: new FormControl('None', Validators.required),
    EnteredBy: new FormControl(''),
    Message: new FormControl(''),
    IsActive: new FormControl(true),
  });

  paymentModes: any = [
    { id: 1, name: 'Bank Transfer' },
    { id: 2, name: 'Cash' },
    { id: 3, name: 'Draft' },
  ];
  selectedPaymentMode: any = 'None';

  constructor(
    private purposeService: ManagePurposeOfTransferService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.subscribeToValueChanges('PurposeOfTransfer_Name');
    this.subscribeToValueChanges('Purpose_Value');
    this.getAllPurposeOfTransfer();
  }

  getAllPurposeOfTransfer(purposeofTransferId = 0) {
    this.purposeService
      .getAllPurposeOfTransfer(
        getCacheData('admin_token'),
        getCacheData('username'),
        purposeofTransferId
      )
      .subscribe({
        next: (res: any) => {
          this.purposeOfTransfers = res;
          if (!this.allPurposeOfTransfers.length) {
            this.allPurposeOfTransfers = [...res];
          }
        },
        error: (err) => {
          this.purposeOfTransfers = [];
        },
        complete: () => {},
      });
  }

  searchPurposeofTransfer() {
    let purposeofTransferId: any = 0;

    if (this.selectedPurposeOfTransfer !== 'All') {
      purposeofTransferId = this.selectedPurposeOfTransfer;
    }

    this.getAllPurposeOfTransfer(purposeofTransferId);
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openPurposeofTransferModal(type: any) {
    if (type == 'add') {
      this.managePurposeofTransferForm.reset({
        BaseTypeID: this.selectedPaymentMode,
      });
    }
    this.type = type;
    this.openModal(this.managePurposeofTransferModal);
  }

  get f() {
    return this.managePurposeofTransferForm.value;
  }

  addPurposeofTransfer() {
    if (!this.managePurposeofTransferForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.BaseTypeID == 'None') {
      alert('Please Select Payment Mode!');
      return;
    }

    let payload;
    payload = this.managePurposeofTransferForm.getRawValue();
    payload = {
      ...payload,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: this.f.EnteredBy || 'none',
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        PurposeOfTransfer_ID: this.f.PurposeOfTransfer_ID,
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addPurposeofTransfer' : 'updatePurposeofTransfer';

    this.purposeService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllPurposeOfTransfer();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editPurposeofTransfer(purposeOfTransfer: any) {
    const baseTypeID = this.paymentModes.find(
      (p) => p.name == purposeOfTransfer.PaymentMode
    )?.id;
    this.managePurposeofTransferForm.patchValue({
      PurposeOfTransfer_ID: purposeOfTransfer?.ID,
      PurposeOfTransfer_Name: purposeOfTransfer?.Name,
      Purpose_Value: purposeOfTransfer?.Description,
      BaseTypeID: baseTypeID,
    });
    this.openPurposeofTransferModal('edit');
  }

  subscribeToValueChanges(fieldName: string) {
    this.managePurposeofTransferForm
      .get(fieldName)
      ?.valueChanges.subscribe((value) => {
        if (value) {
          const updatedValue = value.toUpperCase();
          this.managePurposeofTransferForm.patchValue(
            { [fieldName]: updatedValue },
            { emitEvent: false }
          );
        }
      });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
