import { Component, TemplateRef, ViewChild } from '@angular/core';
import { SendingAgentMarginService } from '../services/sending-agent-margin.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { count } from 'rxjs';

@Component({
  selector: 'app-sending-agent-margin',
  templateUrl: './sending-agent-margin.component.html',
  styleUrls: ['./sending-agent-margin.component.css'],
})
export class SendingAgentMarginComponent {
  @ViewChild('sendingAgent') sendingAgentModal;
  bsModalRef: BsModalRef;

  countryList: any = [];
  selectedCountry: any = '-1';
  agentList: any = [];
  selectedAgent: any = '-1';
  sendCountryAddList: any = [];
  agentAddList: any = [];
  settlementCurrencyList: any = [];
  Agent_ID: any;
  receiveCountry: any;
  receiveCurrency: any;
  countryId: any;
  sendingAgentMargins: any = [];
  type: any = 'add';
  agentCurrency: any;

  sendingAgentMarginForm = new FormGroup({
    PayInCountryId: new FormControl('None', Validators.required),
    Agent_ID: new FormControl('None', Validators.required),
    Agent_CurrencyID: new FormControl('None', Validators.required),
    Target_CountryID: new FormControl('None', Validators.required),
    Target_CurrencyID: new FormControl('None', Validators.required),
    Margin_Rate: new FormControl('', Validators.required),
    UserID: new FormControl('', Validators.required),
    Agent_Currency: new FormControl('None', Validators.required),
    Receive_Country: new FormControl('None', Validators.required),
    Receive_Currency: new FormControl('None', Validators.required),
  });

  constructor(
    private sendingAgentService: SendingAgentMarginService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendCountryList();
    this.getSendCountryAddList();
  }

  getSendCountryList() {
    this.sendingAgentService.getSendCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.countryId = countryId;
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sendingAgentService.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: (err) => {
        this.agentList = [];
      },
    });
  }

  filter() {
    if (this.selectedCountry != '-1' && this.selectedAgent != '-1') {
      this.sendingAgentService
        .getFillGrid(this.selectedCountry, this.selectedAgent)
        .subscribe({
          next: (res: any) => {
            this.sendingAgentMargins = res;
          },
          error: (err) => {
            this.sendingAgentMargins = [];
          },
        });
    }
  }

  getSendCountryAddList() {
    this.sendingAgentService.getSendCountryAddList().subscribe({
      next: (res: any) => {
        this.sendCountryAddList = res;
      },
      error: (err) => {
        this.sendCountryAddList = [];
      },
    });
  }

  onCountryAddChange(countryId: any) {
    if (countryId) {
      this.countryId = countryId;
      this.getSendAgentAddList(countryId);
      this.getReceiveCountry(this.countryId);
      this.getReceiveCurrency(this.countryId);
    }
  }
  getReceiveCountry(countryId) {
    this.sendingAgentService.getReceiveCountry(countryId).subscribe({
      next: (res: any) => {
        this.receiveCountry = res;
        this.sendingAgentMarginForm.patchValue({
          Receive_Country: res?.[0]?.CountryID,
        });
        console.log('res?.[0]?.CountryID: ', res?.[0]?.CountryID);
      },
      error: (err) => {
        throw err;
      },
    });
  }

  getSendAgentAddList(sendCountryId: any = 0) {
    this.sendingAgentService.getSendAgentAddList(sendCountryId).subscribe({
      next: (res: any) => {
        this.agentAddList = res;
        this.sendingAgentMarginForm.patchValue({
          Agent_ID: res?.[0]?.Agent_ID,
        });
      },
      error: (err) => {
        this.agentAddList = [];
      },
      complete: () => {
        this.getAgentCurrency(this.countryId, this.Agent_ID);
      },
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSendingAgentMarginModal(type: any) {
    if (type == 'add') {
      this.sendingAgentMarginForm.reset({
        PayInCountryId: 'None',
      });
    }
    this.type = type;
    this.openModal(this.sendingAgentModal);
  }

  get f() {
    return this.sendingAgentMarginForm.value;
  }

  onChangeStatus(event: any) {
    this.Agent_ID = event;
    this.getAgentCurrency(this.countryId, this.Agent_ID);
  }

  onChangeCountry(event: any) {
    this.getReceiveCurrency(event);
    console.log('event: ', event);
  }

  getReceiveCurrency(countryId) {
    this.sendingAgentService.getReceiveCurrency(countryId).subscribe({
      next: (res: any) => {
        this.receiveCurrency = res;
        this.sendingAgentMarginForm.patchValue({
          Receive_Currency: res?.[0]?.Currency_Id,
        });
      },
      error: (err) => {
        throw err;
      },
    });
  }
  getAgentCurrency(CountryId, Agent_ID) {
    this.sendingAgentService.getAgentCurrency(CountryId, Agent_ID).subscribe({
      next: (res: any) => {
        this.agentCurrency = res;
        this.sendingAgentMarginForm.patchValue({
          Agent_Currency: res?.[0]?.Currency_Id,
        });
        console.log('res: ', res);
      },
      error: (err) => {
        throw err;
      },
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
