import { Component, TemplateRef, ViewChild } from '@angular/core';
import { SendingPercentageWiseService } from '../services/sending-percentage-wise.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { CountryWiseCommissionService } from '../services/country-wise-commission.service';
import { getCacheData } from '../utils/helper';

@Component({
  selector: 'app-sending-percentage-wise',
  templateUrl: './sending-percentage-wise.component.html',
  styleUrls: ['./sending-percentage-wise.component.css'],
})
export class SendingPercentageWiseComponent {
  @ViewChild('sendingPercentage') sendingPercentageModal;
  bsModalRef: BsModalRef;

  countryList: any = [];
  selectedCountry: any = '0';
  agentList: any = [];
  selectedAgent: any = '0';

  sendingPercentageWise: any = [];
  type: any = 'add';

  countryInsertList: any = [];
  sendCurrencyList: any = [];
  sendAgentInsertList: any = [];
  receiveCountryList: any = [];
  receiveCurrencyList: any = [];
  agentAccountList: any = [];
  modeofPaymentList: any = [];
  commissionTypeList: any = [];

  endRangeError: boolean = false;

  sendingPercentageForm = new FormGroup({
    ID: new FormControl(),
    PayInCountryID: new FormControl('None', Validators.required),
    PayInCurrencyID: new FormControl('None', Validators.required),
    AgentID: new FormControl('None', Validators.required),
    PayOutCountryID: new FormControl('None', Validators.required),
    PayOutCurrencyID: new FormControl('None', Validators.required),
    ToAgentID: new FormControl('None', Validators.required),
    ModeOfPaymentID: new FormControl('None', Validators.required),
    StartRange: new FormControl('', Validators.required),
    EndRange: new FormControl('', Validators.required),
    Percentage: new FormControl('', Validators.required),
    PayCommissionType: new FormControl('', Validators.required),
    MinimumCommission: new FormControl('', Validators.required),
    MaximumCommission: new FormControl('', Validators.required),
  });

  constructor(
    private sendingPercentageService: SendingPercentageWiseService,
    private countryWiseService: CountryWiseCommissionService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendCountryList();
    this.getInsertSendCountry();
    this.getTotalCommissionType();
  }

  getSendCountryList() {
    this.sendingPercentageService.getSendCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sendingPercentageService.getSendAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  filter() {
    this.sendingPercentageService
      .getFillGrid(this.selectedCountry, this.selectedAgent)
      .subscribe({
        next: (res: any) => {
          this.sendingPercentageWise = res;
          this.formatSendingPercentageWise(this.sendingPercentageWise);
        },
        error: (err) => {
          this.sendingPercentageWise = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSendingPercentageWiseModal(type: any) {
    if (type == 'add') {
      this.sendingPercentageForm.reset({
        PayInCountryID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.sendingPercentageModal);
  }

  get f() {
    return this.sendingPercentageForm.value;
  }

  getInsertSendCountry() {
    this.countryWiseService
      .getInsertSendCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.countryInsertList = res;
        },
        error: (err) => {
          this.countryInsertList = [];
        },
        complete: () => {},
      });
  }

  onCountryInsertChange(countryId) {
    if (countryId != 'None') {
      this.getInsertSendCurrency(countryId);
      this.getReceiveCountryList();
    }
  }

  getInsertSendCurrency(payInCountryId: any = 0) {
    this.countryWiseService
      .getInsertSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendCurrencyList = res;
          this.sendingPercentageForm.patchValue({
            PayInCurrencyID: 'None',
          });
        },
        error: (err) => {
          this.sendCurrencyList = [];
        },
        complete: () => {},
      });
  }

  onSendCurrencyChange(currencyId) {
    if (currencyId != 'None' && this.f.PayInCountryID != 'None') {
      this.getSendAgentInsertList(currencyId, this.f.PayInCountryID);
    }
  }

  getSendAgentInsertList(currencyId: any = 0, countryId: any = 0) {
    this.sendingPercentageService
      .getSendAgentInsertList(currencyId, countryId)
      .subscribe({
        next: (res: any) => {
          this.sendAgentInsertList = res;
          this.sendingPercentageForm.patchValue({
            AgentID: 'None',
          });
        },
        error: (err) => {
          this.sendAgentInsertList = [];
        },
        complete: () => {},
      });
  }

  getReceiveCountryList(countryId: any = 0) {
    this.sendingPercentageService.getReceiveCountryList(countryId).subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
        this.sendingPercentageForm.patchValue({
          PayOutCountryID: 'None',
        });
      },
      error: (err) => {
        this.receiveCountryList = [];
      },
      complete: () => {},
    });
  }

  onReceiveCountryChange(countryId) {
    if (countryId != 'None') {
      this.getReceiveCurrencyList(countryId);
    }
  }

  getReceiveCurrencyList(countryId: any = 0) {
    this.sendingPercentageService.getReceiveCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.receiveCurrencyList = res;
        this.sendingPercentageForm.patchValue({
          PayOutCurrencyID: 'None',
        });
      },
      error: (err) => {
        this.receiveCurrencyList = [];
      },
      complete: () => {},
    });
  }

  onReceiveCurrencyChange(currencyId) {
    if (currencyId != 'None' && this.f.PayOutCountryID != 'None') {
      this.getAgentAccountList(this.f.PayOutCountryID, currencyId);
    }
  }

  getAgentAccountList(countryId: any = 0, currencyId: any = 0) {
    this.sendingPercentageService
      .getAgentAccountList(countryId, currencyId)
      .subscribe({
        next: (res: any) => {
          this.agentAccountList = res;
          this.sendingPercentageForm.patchValue({
            ToAgentID: 'None',
          });
        },
        error: (err) => {
          this.agentAccountList = [];
        },
        complete: () => {},
      });
  }

  onReceiveAgentChange(agentId) {
    if (agentId != 'None' && this.f.PayOutCountryID != 'None') {
      this.getModeofPaymentList(this.f.PayOutCountryID, agentId);
    }
  }

  getModeofPaymentList(countryId: any = 0, receiveAgentId: any = 0) {
    this.sendingPercentageService
      .getModeofPaymentList(countryId, receiveAgentId)
      .subscribe({
        next: (res: any) => {
          this.modeofPaymentList = res;
          this.sendingPercentageForm.patchValue({
            ModeOfPaymentID: 'None',
          });
        },
        error: (err) => {
          this.modeofPaymentList = [];
        },
        complete: () => {},
      });
  }

  getTotalCommissionType() {
    this.countryWiseService
      .getTotalCommissionType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.commissionTypeList = res;
          this.sendingPercentageForm.patchValue({
            PayCommissionType: 'None',
          });
        },
        error: (err) => {
          this.commissionTypeList = [];
        },
        complete: () => {},
      });
  }

  allowOneDecimal(event: Event, range: string): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;

    this.checkEndRange(range);
  }

  checkEndRange(rangeType: string): void {
    const startRange = this.sendingPercentageForm.get('StartRange').value;
    const endRange = this.sendingPercentageForm.get('EndRange').value;

    switch (rangeType) {
      case 'EndRange':
        this.endRangeError = Number(endRange) < Number(startRange);
        break;

      default:
        break;
    }
  }

  addSendingPercentage() {
    if (!this.sendingPercentageForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (
      !this.checkSelection(this.f.PayInCountryID, 'Please Select Send Country.')
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayInCurrencyID,
        'Please Select Send Currency.'
      )
    ) {
      return;
    }

    if (!this.checkSelection(this.f.AgentID, 'Please Select Sending Agent.')) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayOutCountryID,
        'Please Select Receive Country.'
      )
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayOutCurrencyID,
        'Please Select Receive Currency.'
      )
    ) {
      return;
    }

    if (
      !this.checkSelection(this.f.ToAgentID, 'Please Select Receiving Agent.')
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.ModeOfPaymentID,
        'Please Select Mode of Payment.'
      )
    ) {
      return;
    }

    let payload;
    payload = {
      ID: 0,
      PayInCountryID: this.f.PayInCountryID,
      PayInCurrencyID: this.f.PayInCurrencyID,
      AgentID: this.f.AgentID,
      PayOutCountryID: this.f.PayOutCountryID,
      PayOutCurrencyID: this.f.PayOutCurrencyID,
      ToAgentID: this.f.ToAgentID,
      ModeOfPaymentID: this.f.ModeOfPaymentID,
      StartRange: this.f.StartRange,
      EndRange: this.f.EndRange,
      Percentage: this.f.Percentage,
      PayCommissionType: this.f.PayCommissionType,
      MinimumCommission: this.f.MinimumCommission,
      MaximumCommission: this.f.MaximumCommission,
      EnteredBy: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: Number(this.f.ID),
      };
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addSendingPercentageWise'
        : 'updateSendingPercentageWise';

    this.sendingPercentageService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        alert(
          'Sending Agent Percentage Wise Commission Slab information added successfully.'
        );
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  checkSelection(value: string, errorMessage: string): boolean {
    if (value == 'None') {
      alert(errorMessage);
      return false;
    }
    return true;
  }

  formatSendingPercentageWise(array: any = []) {
    for (let index = 0; index < array.length; index++) {
      const element = array[index];
      if (element?.StartRange !== null && element?.StartRange !== undefined) {
        element.StartRange = this.formatWithThreeDecimalPlaces(
          element.StartRange
        );
      }
      if (element?.EndRange !== null && element?.EndRange !== undefined) {
        element.EndRange = this.formatWithThreeDecimalPlaces(element.EndRange);
      }
      if (element?.Percentage !== null && element?.Percentage !== undefined) {
        element.Percentage = this.formatWithThreeDecimalPlaces(
          element.Percentage
        );
      }
      if (
        element?.MinimumCommission !== null &&
        element?.MinimumCommission !== undefined
      ) {
        element.MinimumCommission = this.formatWithThreeDecimalPlaces(
          element.MinimumCommission
        );
      }
      if (
        element?.MaximumCommission !== null &&
        element?.MaximumCommission !== undefined
      ) {
        element.MaximumCommission = this.formatWithThreeDecimalPlaces(
          element.MaximumCommission
        );
      }
    }
  }

  formatWithThreeDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 3 decimal places
      return `${integerPart}.${fractionalPart.padEnd(3, '0').slice(0, 3)}`;
    } else {
      // If no decimal places, add 3 zeros
      return `${integerPart}.000`;
    }
  }

  reset() {
    this.sendingPercentageForm.reset({
      PayInCountryID: 'None',
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
