import { Component } from '@angular/core';
import { SendingSubAgentCommissionService } from '../services/sending-sub-agent-commission.service';
import { getCacheData } from '../utils/helper';
import { CountryWiseCommissionService } from '../services/country-wise-commission.service';

@Component({
  selector: 'app-sending-sub-agent-commission',
  templateUrl: './sending-sub-agent-commission.component.html',
  styleUrls: ['./sending-sub-agent-commission.component.css'],
})
export class SendingSubAgentCommissionComponent {
  countryList: any = [];
  selectedCountry: any;
  agentList: any = [];
  selectedAgent: any;
  subAgentList: any = [];
  selectedSubAgent: any;

  sendingSubAgentCommission: any = [];
  commissionTypeList: any = [];
  selectedCommissionType: any;

  constructor(
    private sendingSubAgentService: SendingSubAgentCommissionService,
    private countryWiseService: CountryWiseCommissionService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getTotalCommissionType();
  }

  getCountryList() {
    this.sendingSubAgentService.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountry = res?.[0]?.CountryId;
        this.getAgentList(this.selectedCountry);
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sendingSubAgentService.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgent = res?.[0]?.Agent_ID;
        this.getSubAgentList(this.selectedAgent);
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId) {
      this.getSubAgentList(agentId);
    }
  }

  getSubAgentList(agentId: any = 0) {
    this.sendingSubAgentService.getSubAgentList(agentId).subscribe({
      next: (res: any) => {
        this.subAgentList = res;
        this.selectedSubAgent = this.subAgentList.length
          ? res?.[0]?.AgentBranch_ID
          : 'None';
      },
      error: (err) => {
        this.subAgentList = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedAgent && this.selectedSubAgent != 'None') {
      this.sendingSubAgentService
        .getAllRecord(this.selectedAgent, this.selectedSubAgent)
        .subscribe({
          next: (res: any) => {
            this.sendingSubAgentCommission = res;
            this.formatSendingSubAgentCommission(
              this.sendingSubAgentCommission
            );
          },
          error: (err) => {
            this.sendingSubAgentCommission = [];
          },
          complete: () => {},
        });
    }
  }

  formatSendingSubAgentCommission(array: any = []) {
    for (let index = 0; index < array.length; index++) {
      const element = array[index];
      if (
        element?.TotalCommissionValue !== null &&
        element?.TotalCommissionValue !== undefined
      ) {
        element.TotalCommissionValue = this.formatWithThreeDecimalPlaces(
          element.TotalCommissionValue
        );
      }
      if (
        element?.CommissionValue !== null &&
        element?.CommissionValue !== undefined
      ) {
        element.CommissionValue = this.formatWithThreeDecimalPlaces(
          element.CommissionValue
        );
      }
      if (
        element?.SubCommissionValue !== null &&
        element?.SubCommissionValue !== undefined
      ) {
        element.SubCommissionValue = this.formatWithThreeDecimalPlaces(
          element.SubCommissionValue
        );
      }
    }
  }

  formatWithThreeDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 3 decimal places
      return `${integerPart}.${fractionalPart.padEnd(3, '0').slice(0, 3)}`;
    } else {
      // If no decimal places, add 3 zeros
      return `${integerPart}.000`;
    }
  }

  getTotalCommissionType() {
    this.countryWiseService
      .getTotalCommissionType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.commissionTypeList = res;
        },
        error: (err) => {
          this.commissionTypeList = [];
        },
        complete: () => {},
      });
  }

  update() {
    const updatedSubAgentCommission = this.getUpdatedSubAgentCommission();

    this.updatedSubAgentCommission(updatedSubAgentCommission);
  }

  getUpdatedSubAgentCommission() {
    const updatedSubAgentCommission = this.sendingSubAgentCommission
      .filter((con) => con.ID)
      .map((con) => ({
        ID: con?.ID,
        SubAgentID: con?.SubAgentID,
        MasterAgentCommSlabID: con?.MasterCommSlabID,
        Agent_ID: con?.Agent_ID,
        PayInCommissionType: con?.SubPayInCommissionType,
        CommissionValue: con?.SubCommissionValue,
        Action: con?.Action,
        IschkActive: con?.SubCommActive,
        G_UserID: 'string',
      }));

    return updatedSubAgentCommission;
  }

  updatedSubAgentCommission(updatedSubAgentCommission) {
    if (updatedSubAgentCommission?.length) {
      this.sendingSubAgentService
        .updateSubAgentCommission(updatedSubAgentCommission)
        .subscribe({
          next: (res: any) => {
            this.filter();
            alert('Commission Slab updated successfully.');
          },
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
