import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class AgentWiseCommissionService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getSendCountry(token: any, username) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/sending-country`,
      {
        headers: headers,
      }
    );
  }

  getReceiveCountry(token: any, username, payInCountryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receive-country?PayInCountry=${payInCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getModeofPayment(token: any, username) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/mode-Of-payment`,
      {
        headers: headers,
      }
    );
  }

  getSendAgent(token: any, username, payInCountryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/send-agent?PayInCountry=${payInCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getReceiveAgent(token: any, username, payOutCountryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receive-agent?PayOutCountry=${payOutCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getSubAgent(token: any, username, receiveAgentId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/sub-agent?ReceAgentID=${receiveAgentId}`,
      {
        headers: headers,
      }
    );
  }

  getAllAgentWiseCommission(
    token: any,
    username,
    payInCountryId: any = 0,
    payOutCountryId: any = 0,
    agentId: any = 0,
    receiveAgentId: any = 0,
    modeofPaymentId: any = 0,
    subAgentId: any = 0
  ) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/agent-wise-commission?PayInCountryId=${payInCountryId}&PayOutCountryId=${payOutCountryId}&Agent_ID=${agentId}&ReceivingAgent=${receiveAgentId}&ModeOfPayment_ID=${modeofPaymentId}&SubAgentID=${subAgentId}`,
      {
        headers: headers,
      }
    );
  }

  getSendCountryInsert(token: any, username) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/sendcountry-insert`,
      {
        headers: headers,
      }
    );
  }

  getSendAgentInsert(token: any, username, payInCountryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/agent-insert?PayInCountryId=${payInCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getModeofPaymentInsert(token: any, username, agentId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/modeofaymentdropdown-insert?AgentId=${agentId}`,
      {
        headers: headers,
      }
    );
  }

  getSendCurrencyInsert(token: any, username, countryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/sendcurrency-insert?CountryID=${countryId}`,
      {
        headers: headers,
      }
    );
  }

  getReceiveCountryInsert(token: any, username) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receivecountry-insert`,
      {
        headers: headers,
      }
    );
  }

  getReceiveCurrencyInsert(token: any, username, payOutCountryId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receivecurrency-insert?CountryId=${payOutCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getReceiveAgentInsert(
    token: any,
    username,
    receiveCurrencyId: any = 0,
    payOutCountryId: any = 0
  ) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receiveagent-insert?CurrencyID=${receiveCurrencyId}&CountryID=${payOutCountryId}`,
      {
        headers: headers,
      }
    );
  }

  getReceiveSubAgentInsert(token: any, username, agentId: any = 0) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/receivesubagent-insert?AgentId=${agentId}`,
      {
        headers: headers,
      }
    );
  }

  addRates(token: any, username, currencyId, ratePer, payload: any) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/manage-exchange-rates/min-max/add?CurrencyId=${currencyId}&RatePer=${ratePer}&UserID=1
      `,
      payload,
      {
        headers: headers,
      }
    );
  }

  updateRates(token: any, username, payload: any) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/manage-exchange-rates/min-max/update
      `,
      payload,
      {
        headers: headers,
      }
    );
  }

  addAgentWiseCommission(payload: any) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/Add
      `,
      payload,
      {
        headers: headers,
      }
    );
  }

  updateAgentWiseCommission(payload: any) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/update
      `,
      payload,
      {
        headers: headers,
      }
    );
  }

  getAgentWiseCommissionDataById(id: any) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/getdatabyid?ID=${id}`,
      {
        headers: headers,
      }
    );
  }

  approveAgentWiseCommission(moduleID: any = 1, Id: any) {
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/approve?ModuleID=${moduleID}&Id=${Id}
      `,
      {},
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  updateAgentWiseCommissionGrid(payload: any) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/grid-update
      `,
      payload,
      {
        headers: headers,
      }
    );
  }

  deleteAgentWiseCommission(payload: any) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/agent-wise-commission/delete`,
      payload,
      {
        headers: headers,
      }
    );
  }
}
