import { Component, ElementRef, ViewChild } from '@angular/core';
import { ViewCancelTransactionService } from '../services/view-cancel-transaction.service';
import * as XLSX from 'xlsx';
import { formatDate } from '@angular/common';

@Component({
  selector: 'app-view-cancel-transaction',
  templateUrl: './view-cancel-transaction.component.html',
  styleUrls: ['./view-cancel-transaction.component.css'],
})
export class ViewCancelTransactionComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any = -1;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 0;
  transactionNo: any = '';

  startDate: any;
  endDate: any;
  cancelledType: any = [
    {
      id: 0,
      type: 'All',
    },
    {
      id: 10,
      type: 'CancelWithOutCharges',
    },
    {
      id: 11,
      type: 'CancelWithCharges',
    },
    {
      id: 14,
      type: 'Remove Transaction',
    },
  ];
  selectedCancelledType: any = 0;

  statusList: any = [
    {
      id: -1,
      name: 'All',
    },
    {
      id: 0,
      name: 'Pending',
    },
    {
      id: 1,
      name: 'Processed',
    },
  ];
  selectedStatusList: any = -1;

  allCancelTransactions: any = [];
  selectedFileFormat: any = '1';

  tableCancelledType: any = [
    {
      id: 10,
      type: 'Cancellation / Refund charges',
    },
    {
      id: 11,
      type: 'Cancellation / do not refund charges',
    },
    {
      id: 0,
      type: 'Delete Cancellation Request',
    },
    {
      id: 29,
      type: 'MoveToInProcess',
    },
  ];

  selectedTableCancelledType: any = 10;

  constructor(private viewCancelTransactionSvc: ViewCancelTransactionService) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentAccountList(-1);
    this.getAgentBranchList(0);
  }

  getCountryList() {
    this.viewCancelTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: () => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentAccountList(countryId);
    }
  }

  getAgentAccountList(countryId: any = 0) {
    this.viewCancelTransactionSvc.getAgentAccountList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId != -1) {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.viewCancelTransactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }
    let payload: any = {};
    payload = {
      TransactionNo: this.transactionNo,
    };
    this.viewCancelTransactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.allCancelTransactions = res;
      },
      error: (err) => {
        this.allCancelTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.startDate && !this.endDate) {
      alert('Please Select Start & End Date!');
      return;
    }

    let countryId = -1;
    if (this.selectedCountryList != 0) {
      countryId = this.selectedCountryList;
    }
    let payload: any = {};
    payload = {
      TransactionNo: '',
      DdlCountryID: countryId,
      DdlAgentsID: this.selectedAgentList,
      DrpAgentBranch: this.selectedAgentBranchList,
      StartDate: this.startDate,
      EndDate: this.endDate,
      DrpCancelledTypeID: this.selectedCancelledType,
      DrpStatusID: this.selectedStatusList,
      SubUserGroup_Id: 0,
    };
    this.viewCancelTransactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.allCancelTransactions = res;
      },
      error: (err) => {
        this.allCancelTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_Cancel_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  get totalUSDAmount() {
    let totalAmount = 0;
    if (this.allCancelTransactions?.length) {
      for (let i = 0; i < this.allCancelTransactions.length; i++) {
        const element = this.allCancelTransactions[i];
        totalAmount += element.USDAmount;
      }
    }
    return totalAmount;
  }

  cancelTransaction(item) {
    console.log('item: ', item);
    const CancelReqDate =
      item?.CancelReqDate instanceof Date ? item.CancelReqDate : new Date();

    // Format the time using Intl.DateTimeFormat
    const time = new Intl.DateTimeFormat('en-US', {
      hour: 'numeric',
      minute: 'numeric',
    }).format(CancelReqDate);

    const payload = {
      TransactionNo: item?.TransactionNumber,
      App_ID: item?.App_ID,
      TrStatusID: this.selectedTableCancelledType,
      txtRemarks: 'None',
      UserID: 1,
      Date: item?.CancelReqDate,
      Time: time,
    };
    this.viewCancelTransactionSvc.CancelTransaction(payload).subscribe({
      next: (res: any) => {
        alert(res);
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
