import * as XLSX from 'xlsx';
import {
  Component,
  OnInit,
  ViewChild,
  TemplateRef,
  ElementRef,
} from '@angular/core';
import { ViewComplainceHoldListService } from '../services/view-complaince-hold-list.service';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { Router } from '@angular/router';

@Component({
  selector: 'app-view-complaince-hold-list',
  templateUrl: './view-complaince-hold-list.component.html',
  styleUrls: ['./view-complaince-hold-list.component.css'],
})
export class ViewComplainceHoldListComponent implements OnInit {
  @ViewChild('viewTransactionLog') viewTransactionLog: TemplateRef<any>;
  @ViewChild('viewCompliance') viewComplianceModal: TemplateRef<any>;
  @ViewChild('complianceTransactionTable')
  complianceTransactionTable: ElementRef;
  bsModalRef: BsModalRef;
  records: any[] = [];
  complianceLogs: any;
  viewComplianceList: any = [];
  transactionNo: any = '';
  selectedRecord: any;
  allComplianceTransactions: any = [];
  allRecordList: any = [
    {
      id: -1,
      value: 'All',
    },
    {
      id: 10,
      value: '10 transactions',
    },
    {
      id: 25,
      value: '25 transactions',
    },
    {
      id: 50,
      value: '50 transactions',
    },
    {
      id: 100,
      value: '100 transactions',
    },
  ];

  constructor(
    private holdListService: ViewComplainceHoldListService,
    private modalService: BsModalService,
    private router: Router
  ) {}

  ngOnInit() {
    this.allComplianceTransactions = [];
    this.fetchData();
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  navigateToComplianceHoldTransaction(transactionNo: any) {
    this.router.navigate([
      `/panel/complaince-hold-transaction-process/${transactionNo}`,
    ]);
  }
  openComplianceHoldLogModal(transactionNo: any) {
    this.bsModalRef = this.modalService.show(this.viewTransactionLog, {
      class: 'modal-lg',
    });

    this.holdListService.viewComplianceHoldListLog(transactionNo).subscribe(
      (data: any) => {
        console.log('Response from server:', data);
        this.complianceLogs = data;
      },
      (error) => {
        console.error('Error fetching data:', error);
      }
    );
  }

  closeComplianceHoldLogModal() {
    this.bsModalRef.hide();
  }

  fetchData() {
    let topRecords: number;

    if (this.selectedRecord === 'All') {
      topRecords = 0;
    } else {
      topRecords = +this.selectedRecord;
    }

    this.holdListService.fetchComplianceHoldListRecords(topRecords).subscribe(
      (data: any[]) => {
        console.log('Fetched data:', data);
        this.records = data;
      },
      (error) => {
        console.error('Error fetching data:', error);
      }
    );
  }

  fetchDataForSelectedRecord() {}

  displayData() {
    this.fetchData();
  }

  openComplianceModal(transactionNumber: any) {
    this.bsModalRef = this.modalService.show(this.viewComplianceModal, {
      class: 'modal-lg',
    });

    this.holdListService.viewComplianceHoldList(transactionNumber).subscribe(
      (response) => {
        console.log(response);
        this.viewComplianceList = response;
      },
      (error) => {
        console.error(error);
      }
    );
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }

    this.holdListService.filterdByRecords(this.transactionNo).subscribe(
      (data: any[]) => {
        console.log('Filtered data:', data);
        this.records = data;
      },
      (error) => {
        console.error('Error fetching filtered data:', error);
      }
    );
  }

  exportToExcel() {
    console.log('Export button clicked');
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.complianceTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_High_Value_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  transformData() {
    this.allComplianceTransactions = this.allComplianceTransactions.map(
      (item) => {
        const newItem = {};
        for (const key in item) {
          if (item.hasOwnProperty(key)) {
            const newKey = key.replace(/^\|(?<=,)\|\*(?=,|$)|\s+/g, ''); // Remove asterisk and replace spaces with underscores
            newItem[newKey] = item[key];
          }
        }
        return newItem;
      }
    );
  }
}
